@@ -1,15 +1,13 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// Copyright ijl (2022-2026), Anders Kaseorg (2023)
 
 use super::ffi::{
     YYJSON_READ_SUCCESS, yyjson_alc, yyjson_alc_pool_init, yyjson_doc, yyjson_read_err,
     yyjson_read_opts, yyjson_val,
 };
 use crate::deserialize::DeserializeError;
-use crate::deserialize::pyobject::{
-    get_unicode_key, parse_f64, parse_false, parse_i64, parse_none, parse_true, parse_u64,
-};
-use crate::str::PyStr;
-use crate::util::usize_to_isize;
+use crate::deserialize::pyobject::get_unicode_key;
+use crate::ffi::{PyBoolRef, PyDictRef, PyFloatRef, PyIntRef, PyListRef, PyNoneRef, PyStrRef};
 use core::ffi::c_char;
 use core::ptr::{NonNull, null, null_mut};
 use std::borrow::Cow;
@@ -107,7 +105,9 @@ pub(crate) fn deserialize(
     if doc.is_null() {
         ffi!(PyMem_Free(buffer_ptr));
         let msg: Cow<str> = unsafe { core::ffi::CStr::from_ptr(err.msg).to_string_lossy() };
-        return Err(DeserializeError::from_yyjson(msg, err.pos as i64, data));
+        #[allow(clippy::cast_possible_wrap)]
+        let pos = err.pos as i64;
+        return Err(DeserializeError::from_yyjson(msg, pos, data));
     }
     let val = yyjson_doc_get_root(doc);
     let pyval = {
@@ -118,25 +118,23 @@ pub(crate) fn deserialize(
                 ElementType::Uint64 => parse_yy_u64(val),
                 ElementType::Int64 => parse_yy_i64(val),
                 ElementType::Double => parse_yy_f64(val),
-                ElementType::Null => parse_none(),
-                ElementType::True => parse_true(),
-                ElementType::False => parse_false(),
+                ElementType::Null => PyNoneRef::none().as_non_null_ptr(),
+                ElementType::True => PyBoolRef::pytrue().as_non_null_ptr(),
+                ElementType::False => PyBoolRef::pyfalse().as_non_null_ptr(),
                 ElementType::Array | ElementType::Object => unreachable_unchecked!(),
             }
         } else if is_yyjson_tag!(val, TAG_ARRAY) {
-            let pyval = nonnull!(ffi!(PyList_New(usize_to_isize(unsafe_yyjson_get_len(val)))));
+            let pyval = PyListRef::with_capacity(unsafe_yyjson_get_len(val));
             if unsafe_yyjson_get_len(val) > 0 {
-                populate_yy_array(pyval.as_ptr(), val);
+                populate_yy_array(pyval.clone(), val);
             }
-            pyval
+            pyval.as_non_null_ptr()
         } else {
-            let pyval = nonnull!(ffi!(_PyDict_NewPresized(usize_to_isize(
-                unsafe_yyjson_get_len(val)
-            ))));
+            let pyval = PyDictRef::with_capacity(unsafe_yyjson_get_len(val));
             if unsafe_yyjson_get_len(val) > 0 {
-                populate_yy_object(pyval.as_ptr(), val);
+                populate_yy_object(pyval.clone(), val);
             }
-            pyval
+            pyval.as_non_null_ptr()
         }
     };
     ffi!(PyMem_Free(buffer_ptr));
@@ -174,7 +172,7 @@ impl ElementType {
 
 #[inline(always)]
 fn parse_yy_string(elem: *mut yyjson_val) -> NonNull<crate::ffi::PyObject> {
-    PyStr::from_str(str_from_slice!(
+    PyStrRef::from_str(str_from_slice!(
         (*elem).uni.str_.cast::<u8>(),
         unsafe_yyjson_get_len(elem)
     ))
@@ -183,54 +181,42 @@ fn parse_yy_string(elem: *mut yyjson_val) -> NonNull<crate::ffi::PyObject> {
 
 #[inline(always)]
 fn parse_yy_u64(elem: *mut yyjson_val) -> NonNull<crate::ffi::PyObject> {
-    parse_u64(unsafe { (*elem).uni.u64_ })
+    PyIntRef::from_u64(unsafe { (*elem).uni.u64_ }).as_non_null_ptr()
 }
 
 #[inline(always)]
 fn parse_yy_i64(elem: *mut yyjson_val) -> NonNull<crate::ffi::PyObject> {
-    parse_i64(unsafe { (*elem).uni.i64_ })
+    PyIntRef::from_i64(unsafe { (*elem).uni.i64_ }).as_non_null_ptr()
 }
 
 #[inline(always)]
 fn parse_yy_f64(elem: *mut yyjson_val) -> NonNull<crate::ffi::PyObject> {
-    parse_f64(unsafe { (*elem).uni.f64_ })
-}
-
-macro_rules! append_to_list {
-    ($dptr:expr, $pyval:expr) => {
-        unsafe {
-            core::ptr::write($dptr, $pyval);
-            $dptr = $dptr.add(1);
-        }
-    };
+    PyFloatRef::from_f64(unsafe { (*elem).uni.f64_ }).as_non_null_ptr()
 }
 
 #[inline(never)]
-fn populate_yy_array(list: *mut crate::ffi::PyObject, elem: *mut yyjson_val) {
+fn populate_yy_array(mut list: PyListRef, elem: *mut yyjson_val) {
     unsafe {
         let len = unsafe_yyjson_get_len(elem);
         assume!(len >= 1);
         let mut next = unsafe_yyjson_get_first(elem);
-        let mut dptr = (*list.cast::<crate::ffi::PyListObject>()).ob_item;
 
-        for _ in 0..len {
+        for idx in 0..len {
             let val = next;
             if unsafe_yyjson_is_ctn(val) {
                 cold_path!();
                 next = unsafe_yyjson_get_next_container(val);
                 if is_yyjson_tag!(val, TAG_ARRAY) {
-                    let pyval = ffi!(PyList_New(usize_to_isize(unsafe_yyjson_get_len(val))));
-                    append_to_list!(dptr, pyval);
+                    let pyval = PyListRef::with_capacity(unsafe_yyjson_get_len(val));
+                    list.set(idx, pyval.as_ptr());
                     if unsafe_yyjson_get_len(val) > 0 {
-                        populate_yy_array(pyval, val);
+                        populate_yy_array(pyval.clone(), val);
                     }
                 } else {
-                    let pyval = ffi!(_PyDict_NewPresized(usize_to_isize(unsafe_yyjson_get_len(
-                        val
-                    ))));
-                    append_to_list!(dptr, pyval);
+                    let pyval = PyDictRef::with_capacity(unsafe_yyjson_get_len(val));
+                    list.set(idx, pyval.as_ptr());
                     if unsafe_yyjson_get_len(val) > 0 {
-                        populate_yy_object(pyval, val);
+                        populate_yy_object(pyval.clone(), val);
                     }
                 }
             } else {
@@ -240,19 +226,19 @@ fn populate_yy_array(list: *mut crate::ffi::PyObject, elem: *mut yyjson_val) {
                     ElementType::Uint64 => parse_yy_u64(val),
                     ElementType::Int64 => parse_yy_i64(val),
                     ElementType::Double => parse_yy_f64(val),
-                    ElementType::Null => parse_none(),
-                    ElementType::True => parse_true(),
-                    ElementType::False => parse_false(),
+                    ElementType::Null => PyNoneRef::none().as_non_null_ptr(),
+                    ElementType::True => PyBoolRef::pytrue().as_non_null_ptr(),
+                    ElementType::False => PyBoolRef::pyfalse().as_non_null_ptr(),
                     ElementType::Array | ElementType::Object => unreachable_unchecked!(),
                 };
-                append_to_list!(dptr, pyval.as_ptr());
+                list.set(idx, pyval.as_ptr());
             }
         }
     }
 }
 
 #[inline(never)]
-fn populate_yy_object(dict: *mut crate::ffi::PyObject, elem: *mut yyjson_val) {
+fn populate_yy_object(mut dict: PyDictRef, elem: *mut yyjson_val) {
     unsafe {
         let len = unsafe_yyjson_get_len(elem);
         assume!(len >= 1);
@@ -272,18 +258,16 @@ fn populate_yy_object(dict: *mut crate::ffi::PyObject, elem: *mut yyjson_val) {
                 next_key = unsafe_yyjson_get_next_container(val);
                 next_val = next_key.add(1);
                 if is_yyjson_tag!(val, TAG_ARRAY) {
-                    let pyval = ffi!(PyList_New(usize_to_isize(unsafe_yyjson_get_len(val))));
-                    pydict_setitem!(dict, pykey.as_ptr(), pyval);
+                    let pyval = PyListRef::with_capacity(unsafe_yyjson_get_len(val));
+                    dict.set(pykey, pyval.as_ptr());
                     if unsafe_yyjson_get_len(val) > 0 {
                         populate_yy_array(pyval, val);
                     }
                 } else {
-                    let pyval = ffi!(_PyDict_NewPresized(usize_to_isize(unsafe_yyjson_get_len(
-                        val
-                    ))));
-                    pydict_setitem!(dict, pykey.as_ptr(), pyval);
+                    let pyval = PyDictRef::with_capacity(unsafe_yyjson_get_len(val));
+                    dict.set(pykey, pyval.as_ptr());
                     if unsafe_yyjson_get_len(val) > 0 {
-                        populate_yy_object(pyval, val);
+                        populate_yy_object(pyval.clone(), val);
                     }
                 }
             } else {
@@ -294,12 +278,12 @@ fn populate_yy_object(dict: *mut crate::ffi::PyObject, elem: *mut yyjson_val) {
                     ElementType::Uint64 => parse_yy_u64(val),
                     ElementType::Int64 => parse_yy_i64(val),
                     ElementType::Double => parse_yy_f64(val),
-                    ElementType::Null => parse_none(),
-                    ElementType::True => parse_true(),
-                    ElementType::False => parse_false(),
+                    ElementType::Null => PyNoneRef::none().as_non_null_ptr(),
+                    ElementType::True => PyBoolRef::pytrue().as_non_null_ptr(),
+                    ElementType::False => PyBoolRef::pyfalse().as_non_null_ptr(),
                     ElementType::Array | ElementType::Object => unreachable_unchecked!(),
                 };
-                pydict_setitem!(dict, pykey.as_ptr(), pyval.as_ptr());
+                dict.set(pykey, pyval.as_ptr());
             }
         }
     }
