@@ -1,12 +1,13 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// Copyright ijl (2020-2026), Eric Jolibois (2021)
 
 mod backend;
 #[cfg(not(Py_GIL_DISABLED))]
 mod cache;
 mod deserializer;
 mod error;
+mod input;
 mod pyobject;
-mod utf8;
 
 #[cfg(not(Py_GIL_DISABLED))]
 pub(crate) use cache::{KEY_MAP, KeyMap};
