@@ -1,22 +1,23 @@
-// SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// SPDX-License-Identifier: MPL-2.0
+// Copyright ijl (2020-2026)
 
 use core::ffi::c_char;
 
-use core::ptr::null_mut;
-use pyo3_ffi::{
-    Py_DECREF, Py_INCREF, Py_SIZE, Py_TPFLAGS_DEFAULT, PyErr_SetObject, PyExc_TypeError, PyObject,
-    PyType_Ready, PyType_Type, PyTypeObject, PyUnicode_FromStringAndSize, PyVarObject,
+use crate::ffi::{
+    Py_DECREF, Py_INCREF, Py_TPFLAGS_DEFAULT, PyErr_SetObject, PyExc_TypeError, PyObject,
+    PyTupleRef, PyType_Ready, PyType_Type, PyTypeObject, PyUnicode_FromStringAndSize, PyVarObject,
 };
+use core::ptr::null_mut;
 
 #[cfg(Py_GIL_DISABLED)]
 use super::atomiculong::AtomicCULong;
 #[cfg(Py_GIL_DISABLED)]
-use std::sync::atomic::{AtomicIsize, AtomicU32};
+use core::sync::atomic::{AtomicIsize, AtomicU32};
 
 #[cfg(Py_GIL_DISABLED)]
 macro_rules! pymutex_new {
     () => {
-        unsafe { std::mem::zeroed() }
+        unsafe { core::mem::zeroed() }
     };
 }
 
@@ -51,7 +52,7 @@ fn raise_args_exception() {
     unsafe {
         let msg = "orjson.Fragment() takes exactly 1 positional argument";
         let err_msg =
-            PyUnicode_FromStringAndSize(msg.as_ptr().cast::<c_char>(), msg.len() as isize);
+            PyUnicode_FromStringAndSize(msg.as_ptr().cast::<c_char>(), msg.len().cast_signed());
         PyErr_SetObject(PyExc_TypeError, err_msg);
         Py_DECREF(err_msg);
     };
@@ -66,11 +67,12 @@ pub(crate) unsafe extern "C" fn orjson_fragment_tp_new(
     kwds: *mut PyObject,
 ) -> *mut PyObject {
     unsafe {
-        if Py_SIZE(args) != 1 || !kwds.is_null() {
+        let argsob = PyTupleRef::from_ptr_unchecked(args);
+        if argsob.len() != 1 || !kwds.is_null() {
             raise_args_exception();
             null_mut()
         } else {
-            let contents = crate::ffi::PyTuple_GET_ITEM(args, 0);
+            let contents = argsob.get(0);
             Py_INCREF(contents);
             let obj = Box::new(Fragment {
                 #[cfg(Py_GIL_DISABLED)]
@@ -105,26 +107,20 @@ pub(crate) unsafe extern "C" fn orjson_fragment_tp_new(
 pub(crate) unsafe extern "C" fn orjson_fragment_dealloc(object: *mut PyObject) {
     unsafe {
         Py_DECREF((*object.cast::<Fragment>()).contents);
-        std::alloc::dealloc(object.cast::<u8>(), core::alloc::Layout::new::<Fragment>());
+        crate::ffi::PyMem_Free(object.cast::<core::ffi::c_void>());
     }
 }
 
-#[cfg(Py_GIL_DISABLED)]
-const FRAGMENT_TP_FLAGS: AtomicCULong =
-    AtomicCULong::new(Py_TPFLAGS_DEFAULT | pyo3_ffi::Py_TPFLAGS_IMMUTABLETYPE);
-
-#[cfg(all(Py_3_10, not(Py_GIL_DISABLED)))]
-const FRAGMENT_TP_FLAGS: core::ffi::c_ulong =
-    Py_TPFLAGS_DEFAULT | pyo3_ffi::Py_TPFLAGS_IMMUTABLETYPE;
-
-#[cfg(not(Py_3_10))]
-const FRAGMENT_TP_FLAGS: core::ffi::c_ulong = Py_TPFLAGS_DEFAULT;
-
 #[unsafe(no_mangle)]
 #[cold]
 #[cfg_attr(feature = "optimize", optimize(size))]
 pub(crate) unsafe extern "C" fn orjson_fragmenttype_new() -> *mut PyTypeObject {
     unsafe {
+        #[cfg(Py_GIL_DISABLED)]
+        let tp_flags: AtomicCULong =
+            AtomicCULong::new(Py_TPFLAGS_DEFAULT | pyo3_ffi::Py_TPFLAGS_IMMUTABLETYPE);
+        #[cfg(not(Py_GIL_DISABLED))]
+        let tp_flags: core::ffi::c_ulong = Py_TPFLAGS_DEFAULT | pyo3_ffi::Py_TPFLAGS_IMMUTABLETYPE;
         let ob = Box::new(PyTypeObject {
             ob_base: PyVarObject {
                 ob_base: PyObject {
@@ -156,12 +152,12 @@ pub(crate) unsafe extern "C" fn orjson_fragmenttype_new() -> *mut PyTypeObject {
                 _ob_size_graalpy: 0,
             },
             tp_name: c"orjson.Fragment".as_ptr(),
-            tp_basicsize: core::mem::size_of::<Fragment>() as isize,
+            tp_basicsize: core::mem::size_of::<Fragment>().cast_signed(),
             tp_itemsize: 0,
             tp_dealloc: Some(orjson_fragment_dealloc),
             tp_init: None,
             tp_new: Some(orjson_fragment_tp_new),
-            tp_flags: FRAGMENT_TP_FLAGS,
+            tp_flags: tp_flags,
             // ...
             tp_bases: null_mut(),
             tp_cache: null_mut(),
