@@ -1,6 +1,9 @@
 // SPDX-License-Identifier: (Apache-2.0 OR MIT)
+// Copyright ijl (2018-2026), Ben Sully (2021), Nazar Kostetskyi (2022), Aviram Hassan (2020-2021)
 
-use crate::ffi::{Py_intptr_t, Py_ssize_t, PyObject, PyTypeObject};
+use crate::ffi::{
+    Py_intptr_t, Py_ssize_t, PyListRef, PyObject, PyStrRef, PyTupleRef, PyTypeObject,
+};
 use crate::opt::Opt;
 use crate::serialize::buffer::SmallFixedBuffer;
 use crate::serialize::error::SerializeError;
@@ -8,7 +11,6 @@ use crate::serialize::per_type::{
     DateTimeError, DateTimeLike, DefaultSerializer, Offset, ZeroListSerializer,
 };
 use crate::serialize::serializer::PyObjectSerializer;
-use crate::str::PyStr;
 use crate::typeref::{ARRAY_STRUCT_STR, DESCR_STR, DTYPE_STR, NUMPY_TYPES, load_numpy_types};
 use crate::util::isize_to_usize;
 use core::ffi::{c_char, c_int, c_void};
@@ -196,11 +198,13 @@ impl NumpyArray {
     #[cfg_attr(feature = "optimize", optimize(size))]
     pub fn new(ptr: *mut PyObject, opts: Opt) -> Result<Self, PyArrayError> {
         let capsule = ffi!(PyObject_GetAttr(ptr, ARRAY_STRUCT_STR));
+        debug_assert!(!capsule.is_null());
         let array = unsafe {
             (*capsule.cast::<PyCapsule>())
                 .pointer
                 .cast::<PyArrayInterface>()
         };
+        debug_assert!(!array.is_null());
         if unsafe { (*array).two != 2 } {
             ffi!(Py_DECREF(capsule));
             Err(PyArrayError::Malformed)
@@ -212,8 +216,7 @@ impl NumpyArray {
             Err(PyArrayError::NotNativeEndian)
         } else {
             debug_assert!(unsafe { (*array).nd >= 0 });
-            #[allow(clippy::cast_sign_loss)]
-            let num_dimensions = unsafe { (*array).nd as usize };
+            let num_dimensions = unsafe { (*array).nd.cast_unsigned() as usize };
             if num_dimensions == 0 {
                 ffi!(Py_DECREF(capsule));
                 return Err(PyArrayError::UnsupportedDataType);
@@ -291,10 +294,7 @@ impl NumpyArray {
     }
 
     fn dimensions(&self) -> usize {
-        #[allow(clippy::cast_sign_loss)]
-        unsafe {
-            (*self.array).nd as usize
-        }
+        unsafe { (*self.array).nd.cast_unsigned() as usize }
     }
 
     fn shape(&self) -> &[isize] {
@@ -1247,34 +1247,43 @@ impl NumpyDatetimeUnit {
     fn from_pyobject(ptr: *mut PyObject) -> Self {
         let dtype = ffi!(PyObject_GetAttr(ptr, DTYPE_STR));
         let descr = ffi!(PyObject_GetAttr(dtype, DESCR_STR));
-        let el0 = ffi!(PyList_GET_ITEM(descr, 0));
-        let descr_str = ffi!(PyTuple_GET_ITEM(el0, 1));
-        let uni = unsafe { PyStr::from_ptr_unchecked(descr_str).to_str().unwrap() };
-        if uni.len() < 5 {
-            return Self::NaT;
+        let el0 = unsafe { PyListRef::from_ptr_unchecked(descr).get(0) };
+        let descr_str = unsafe { PyTupleRef::from_ptr_unchecked(el0).get(1) };
+        match PyStrRef::from_ptr(descr_str) {
+            Ok(uni) => {
+                match uni.as_str() {
+                    Some(as_str) => {
+                        if as_str.len() < 5 {
+                            return Self::NaT;
+                        }
+                        // unit descriptions are found at
+                        // https://github.com/numpy/numpy/blob/b235f9e701e14ed6f6f6dcba885f7986a833743f/numpy/core/src/multiarray/datetime.c#L79-L96.
+                        let ret = match &as_str[4..as_str.len() - 1] {
+                            "Y" => Self::Years,
+                            "M" => Self::Months,
+                            "W" => Self::Weeks,
+                            "D" => Self::Days,
+                            "h" => Self::Hours,
+                            "m" => Self::Minutes,
+                            "s" => Self::Seconds,
+                            "ms" => Self::Milliseconds,
+                            "us" => Self::Microseconds,
+                            "ns" => Self::Nanoseconds,
+                            "ps" => Self::Picoseconds,
+                            "fs" => Self::Femtoseconds,
+                            "as" => Self::Attoseconds,
+                            "generic" => Self::Generic,
+                            _ => unreachable!(),
+                        };
+                        ffi!(Py_DECREF(dtype));
+                        ffi!(Py_DECREF(descr));
+                        ret
+                    }
+                    None => Self::NaT,
+                }
+            }
+            Err(_) => Self::NaT,
         }
-        // unit descriptions are found at
-        // https://github.com/numpy/numpy/blob/b235f9e701e14ed6f6f6dcba885f7986a833743f/numpy/core/src/multiarray/datetime.c#L79-L96.
-        let ret = match &uni[4..uni.len() - 1] {
-            "Y" => Self::Years,
-            "M" => Self::Months,
-            "W" => Self::Weeks,
-            "D" => Self::Days,
-            "h" => Self::Hours,
-            "m" => Self::Minutes,
-            "s" => Self::Seconds,
-            "ms" => Self::Milliseconds,
-            "us" => Self::Microseconds,
-            "ns" => Self::Nanoseconds,
-            "ps" => Self::Picoseconds,
-            "fs" => Self::Femtoseconds,
-            "as" => Self::Attoseconds,
-            "generic" => Self::Generic,
-            _ => unreachable!(),
-        };
-        ffi!(Py_DECREF(dtype));
-        ffi!(Py_DECREF(descr));
-        ret
     }
 
     /// Return a `NumpyDatetime64Repr` for a value in array with this unit.
@@ -1391,9 +1400,7 @@ impl DateTimeLike for NumpyDatetime64Repr {
 
     fn nanosecond(&self) -> u32 {
         debug_assert!(self.dt.subsec_nanosecond() >= 0);
-        #[allow(clippy::cast_sign_loss)]
-        let ret = self.dt.subsec_nanosecond() as u32; // stmt_expr_attributes
-        ret
+        self.dt.subsec_nanosecond().cast_unsigned()
     }
 
     fn microsecond(&self) -> u32 {
