@@ -1,13 +1,21 @@
-# SPDX-License-Identifier: (Apache-2.0 OR MIT)
+# SPDX-License-Identifier: MPL-2.0
+# Copyright ijl (2018-2026)
 
 import lzma
 import os
+import sys
 import sysconfig
 from pathlib import Path
 from typing import Any
 
 IS_FREETHREADING = sysconfig.get_config_var("Py_GIL_DISABLED")
 
+SUPPORTS_MEMORYVIEW = sys.implementation == "cpython" and not IS_FREETHREADING
+
+SUPPORTS_BYTEARRAY = not IS_FREETHREADING
+
+SUPPORTS_GETREFCOUNT = sys.implementation == "cpython"
+
 numpy = None  # type: ignore
 if not IS_FREETHREADING:
     try:
